//********************************************************************
/// \file
/// 
/// 
/// 
/// 
//********************************************************************
#include "GLGTProfiler.h"



//////////////////////////////////////////////////
GLGTProfiler_Singleton::GLGTProfiler_Singleton(void) :profilerMode(GLGTFLAGARRAYSIZE_GLGTPROFILERMODE) {
    for(unsigned int i = 0; i < GLGTPROFILER_SAMPLEDATASIZE; i++) {
        profileSamples[i].valid = false;
        profileSamples[i].locked = false;
    }
    validSampleDataCount = 0;
    profilingBeginTime = 0;
    profilingEndTime = 0;
}

//////////////////////////////////////////////////
GLGTProfiler_Singleton::~GLGTProfiler_Singleton() {

}



//////////////////////////////////////////////////
void GLGTProfiler_Singleton::updateProfileStatistics(void) {
    if(profilerMode[GLGTPROFILERMODE_ENABLE] == false) {return;}

    //Ԃ擾
    profilingEndTime = GLGTTime.getTime();
    float loop_time = (float)(profilingEndTime-profilingBeginTime);
    if(loop_time == 0) {loop_time = 1;}

    float new_ratio = loop_time/1000.0f;
    if(new_ratio > 1.0f) {new_ratio = 1.0f;}
    float old_ratio = 1.0f-new_ratio;

    float new_minmaxratio = 0.1f*loop_time/1000.0f;
    if(new_minmaxratio > 1.0f) {new_minmaxratio = 1.0f;}
    float old_minmaxratio = 1.0f-new_minmaxratio;

    for(unsigned int i = 0; i < validSampleDataCount; i++) {
        if(profileSamples[i].locked == true) {continue;}

        float sample_time = (float)(profileSamples[i].totalProfilingTime-profileSamples[i].totalChildrenProfilingTime);
        float percent_time = 100.0f*sample_time/loop_time;


        //XV
        profileSamples[i].avgRate = (percent_time*new_ratio)+(profileSamples[i].avgRate*old_ratio);

        if(profileSamples[i].minRate > percent_time) {profileSamples[i].minRate = percent_time;}
        else {profileSamples[i].minRate = (percent_time*new_minmaxratio)+(profileSamples[i].minRate*old_minmaxratio);}

        if(profileSamples[i].maxRate < percent_time) {profileSamples[i].maxRate = percent_time;}
        else {profileSamples[i].maxRate = (percent_time*new_minmaxratio)+(profileSamples[i].maxRate*old_minmaxratio);}

        /*
        profileSamples[i].avgRate = (percent_time*new_ratio)+(profileSamples[i].avgRate*old_ratio);
        if(profileSamples[i].minRate > percent_time) {profileSamples[i].minRate = percent_time;}
        else {profileSamples[i].minRate = (percent_time*new_ratio)+(profileSamples[i].minRate*old_ratio);}
        if(profileSamples[i].maxRate < percent_time) {profileSamples[i].maxRate = percent_time;}
        else {profileSamples[i].maxRate = (percent_time*new_ratio)+(profileSamples[i].maxRate*old_ratio);}
        */


    }
    profilingBeginTime = profilingEndTime;

}






//////////////////////////////////////////////////
unsigned int  GLGTProfiler_Singleton::beginSampleProfiling(const char* in_samplename) {
    if(profilerMode[GLGTPROFILERMODE_ENABLE] == false) {return GLGTPROFILER_INVALID;}
    unsigned int i;
    for(i = 0; i < validSampleDataCount; i++) {
        if(profileSamples[i].name == in_samplename) {//v
            if(profileSamples[i].locked == true) {return i;}
            profileSamples[i].locked = true;
            profileSamples[i].openCount++;
            profileSamples[i].beginTime = GLGTTime.getTime();
            return i;
        }
    }
    GLGTASSERT(validSampleDataCount < GLGTPROFILER_SAMPLEDATASIZE); // 󂫂ȂƃAEg

    //vȂ̂ŐVɍ쐬
    profileSamples[i].name = in_samplename;
    profileSamples[i].valid = true;
    profileSamples[i].locked = true;
    profileSamples[i].openCount = 1;
    profileSamples[i].totalProfilingTime = 0;
    profileSamples[i].totalChildrenProfilingTime = 0;
    profileSamples[i].avgRate =  50.0f;
    profileSamples[i].minRate =   0.0f;
    profileSamples[i].maxRate = 100.0f;
    profileSamples[i].beginTime = GLGTTime.getTime();
    validSampleDataCount++;
    return i;

}

//////////////////////////////////////////////////
void GLGTProfiler_Singleton::endSampleProfiling(unsigned int in_sampleindex) {
    if(profilerMode[GLGTPROFILERMODE_ENABLE] == false) {return;}
    if(in_sampleindex == GLGTPROFILER_INVALID) {return;}
    GLGTASSERT(in_sampleindex < GLGTPROFILER_SAMPLEDATASIZE);
    GLGTASSERT(profileSamples[in_sampleindex].valid == true);
    if(profileSamples[in_sampleindex].locked == false) {return;}
    profileSamples[in_sampleindex].locked = false;

    unsigned int sampleendtime = GLGTTime.getTime();

    unsigned int parent = GLGTPROFILER_INVALID;
    profileSamples[in_sampleindex].parentsCount = 0;
    for(unsigned int i = 0; i < validSampleDataCount; i++) {
        if(profileSamples[i].locked == true) {
            profileSamples[in_sampleindex].parentsCount++;
            if(parent == GLGTPROFILER_INVALID) {
                parent = i;
            }
            else if(profileSamples[i].beginTime >= profileSamples[parent].beginTime) {
                parent = i;
            }
        }


    }

    if(parent != GLGTPROFILER_INVALID) {
        profileSamples[parent].totalChildrenProfilingTime += sampleendtime-profileSamples[in_sampleindex].beginTime;
    }

    profileSamples[in_sampleindex].totalProfilingTime += sampleendtime-profileSamples[in_sampleindex].beginTime;



}




//////////////////////////////////////////////////
/// \brief 
void GLGTProfiler_Singleton::drawProfile(int in_left, int in_top, unsigned int in_viewportwidth, unsigned int in_viewportheight) {
    if(profilerMode[GLGTPROFILERMODE_ENABLE] == false) {return;}

    //O
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, in_viewportwidth, in_viewportheight, 0.0, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();  
    glPushAttrib(GL_TEXTURE_BIT|GL_ENABLE_BIT);

    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);

    //Ot`
    int right_min;
    int right_avg;
    int right_max;
    int top    = in_top;
    int bottom = in_top+GLGTFontRenderer.getFontHeight();
    int right = in_left+150;
    glDisable(GL_TEXTURE_2D);
    for(unsigned int i = 0; i < validSampleDataCount; i++) {
        right_min = (int)(profileSamples[i].minRate*1.5f)+in_left;
        right_avg = (int)(profileSamples[i].avgRate*1.5f)+in_left;
        right_max = (int)(profileSamples[i].maxRate*1.5f)+in_left;

        glBegin(GL_QUADS);

        glColor4ub(255,255,255,191);
        glVertex3i(in_left  , top   , 0);
        glVertex3i(in_left  , bottom, 0);
        glVertex3i(right_avg, bottom, 0);
        glVertex3i(right_avg, top   , 0);

        glColor4ub(191,191,191,191);
        glVertex3i(right_avg, top   , 0);
        glVertex3i(right_avg, bottom, 0);
        glVertex3i(right    , bottom, 0);
        glVertex3i(right    , top   , 0);

        glColor4ub(0,255,0,255);
        glVertex3i(right_min-1, top   , 0);
        glVertex3i(right_min-1, bottom, 0);
        glVertex3i(right_min+1, bottom, 0);
        glVertex3i(right_min+1, top   , 0);

        glColor4ub(255,0,0,255);
        glVertex3i(right_max-1, top   , 0);
        glVertex3i(right_max-1, bottom, 0);
        glVertex3i(right_max+1, bottom, 0);
        glVertex3i(right_max+1, top   , 0);

        glEnd();

        top      += GLGTFontRenderer.getFontHeight();
        bottom   += GLGTFontRenderer.getFontHeight();
    }

    //eLXg`
    top = in_top;
    GLGTColor4<unsigned char> color(0,0,0,255);
    for(i = 0; i < validSampleDataCount; i++) {
        GLGTFontRenderer.drawLine(profileSamples[i].name,color,in_left,top);
        top += GLGTFontRenderer.getFontHeight();
    }


    //㏈
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glPopAttrib();

}





//////////////////////////////////////////////////
void GLGTProfiler_Singleton::outputToStdout(unsigned int interval) {
    if(profilerMode[GLGTPROFILERMODE_ENABLE] == false) {return;}
    static unsigned int lastoutput = GLGTTime.getTime();
    unsigned int now = GLGTTime.getTime();
    if(now-lastoutput< interval) {return;}
    printf("\n--------------------------------------------------\n");
    for(unsigned int i = 0; i < validSampleDataCount; i++) {
        printf("[%6.2f]:[%6.2f]:[%6.2f]:[%4u]:[%s]\n",profileSamples[i].avgRate,profileSamples[i].minRate,profileSamples[i].maxRate,
            profileSamples[i].openCount,profileSamples[i].name.c_str());
    }
    lastoutput = now;
}




