//********************************************************************
/// \file
/// 
/// 
/// 
/// 
//********************************************************************
#include "GLGTRectBox.h"



//////////////////////////////////////////////////
GLGTRectBox::GLGTRectBox(const GLGTRectEXT* in_rect) : ptrRectBoxRect(in_rect) {
    rectColor.set(255,255,255,255);
    rectTextureCoord.set(0,1,1,0);
}

//////////////////////////////////////////////////
GLGTRectBox::~GLGTRectBox() {
    unsetRectTexture();
}



//////////////////////////////////////////////////
/// \brief eNX`Zbg
void GLGTRectBox::setRectTexture(const std::string& filename) {
    unsetRectTexture();
    rectTexture = GLGTTextureManager.addTexture(filename);
    if(rectTexture.isReferring() == true) {
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glGenTextures(1, &rectTextureID);
        glBindTexture(GL_TEXTURE_2D, rectTextureID);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA4,
            rectTexture.getWidth(), rectTexture.getHeight(),
            0, GL_RGBA, GL_UNSIGNED_BYTE, rectTexture.getPixels());
    }
}
//////////////////////////////////////////////////
/// \brief eNX`Zbg
void GLGTRectBox::setRectTexture(const GLGTTexture& in_texture) {
    unsetRectTexture();
    rectTexture = in_texture;
    if(rectTexture.isReferring() == true) {
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glGenTextures(1, &rectTextureID);
        glBindTexture(GL_TEXTURE_2D, rectTextureID);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA4,
            rectTexture.getWidth(), rectTexture.getHeight(),
            0, GL_RGBA, GL_UNSIGNED_BYTE, rectTexture.getPixels());
    }
}

//////////////////////////////////////////////////
/// \brief eNX`
void GLGTRectBox::unsetRectTexture(void) {
    if(rectTexture.isReferring() == true) {
        rectTexture.unrefer();
        glDeleteTextures(1,&rectTextureID);
    }
}


//////////////////////////////////////////////////
/// \brief ``悷B
void GLGTRectBox::drawRect(unsigned int projectionwidth, unsigned int projectionheight) {
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, projectionwidth, projectionheight, 0.0, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();  
    glPushAttrib(GL_TEXTURE_BIT|GL_ENABLE_BIT);
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
    glColor4ubv(&rectColor[0]);

    if(rectTexture.isReferring() == true) {
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, rectTextureID);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 

        glBegin(GL_QUADS);
        glTexCoord2f(rectTextureCoord.left , rectTextureCoord.top);
        glVertex3i(ptrRectBoxRect->position.x,
            ptrRectBoxRect->position.y,0);
        glTexCoord2f(rectTextureCoord.left , rectTextureCoord.bottom);
        glVertex3i(ptrRectBoxRect->position.x,
            ptrRectBoxRect->position.y+ptrRectBoxRect->dimension.y,0);
        glTexCoord2f(rectTextureCoord.right, rectTextureCoord.bottom);
        glVertex3i(ptrRectBoxRect->position.x+ptrRectBoxRect->dimension.x,
            ptrRectBoxRect->position.y+ptrRectBoxRect->dimension.y,0);
        glTexCoord2f(rectTextureCoord.right, rectTextureCoord.top);
        glVertex3i(ptrRectBoxRect->position.x+ptrRectBoxRect->dimension.x,
            ptrRectBoxRect->position.y,0);
        glEnd();
    }
    else {
        glDisable(GL_TEXTURE_2D);

        glBegin(GL_QUADS);
        glVertex3i(ptrRectBoxRect->position.x,
            ptrRectBoxRect->position.y,0);
        glVertex3i(ptrRectBoxRect->position.x,
            ptrRectBoxRect->position.y+ptrRectBoxRect->dimension.y,0);
        glVertex3i(ptrRectBoxRect->position.x+ptrRectBoxRect->dimension.x,
            ptrRectBoxRect->position.y+ptrRectBoxRect->dimension.y,0);
        glVertex3i(ptrRectBoxRect->position.x+ptrRectBoxRect->dimension.x,
            ptrRectBoxRect->position.y,0);
        glEnd();


    }

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glPopAttrib();

}

