//********************************************************************
/// \file
/// 
/// 
/// 
/// 
//********************************************************************
#include "GLGTTextureManager.h"




/***************************************************
* GLGTTexture
*/
//////////////////////////////////////////////////
GLGTTexture::GLGTTexture(void) {
    clear();
}
//////////////////////////////////////////////////
GLGTTexture::GLGTTexture(const GLGTTexture& in) {
    clear();
    refer(in.ptrTextureData);
}
//////////////////////////////////////////////////
GLGTTexture::GLGTTexture(GLGTTextureData* in_texdata) {
    clear();
    refer(in_texdata);
}

//////////////////////////////////////////////////
GLGTTexture::~GLGTTexture() {
    unrefer();
}





//////////////////////////////////////////////////
void GLGTTexture::refer(GLGTTextureData* in_texdata) {
    unrefer();
    if(in_texdata != NULL) {
        ptrTextureData = in_texdata;
        ptrTextureData->referenceCount++;
    }
}



//////////////////////////////////////////////////
void GLGTTexture::unrefer(void) {
    if(ptrTextureData != NULL) {
        ptrTextureData->referenceCount--;
        ptrTextureData = NULL;
    }

}


//////////////////////////////////////////////////
GLGTTexture& GLGTTexture::operator =(const GLGTTexture& in) {
    refer(in.ptrTextureData);
    return *this;
}




//////////////////////////////////////////////////
void GLGTTexture::clear(void) {
    ptrTextureData = NULL;

}

















/***************************************************
* GLGTTextureManager_Singleton
*/
//////////////////////////////////////////////////
// RXgN^
GLGTTextureManager_Singleton::GLGTTextureManager_Singleton(void) {
    
    ptrTextureData.resize(GLGTDEFAULTTEXTUREDATAARRAYSIZE, NULL);

}

//////////////////////////////////////////////////
// fXgN^
GLGTTextureManager_Singleton::~GLGTTextureManager_Singleton() {
    std::vector<GLGTTextureData*>::size_type size = ptrTextureData.size();
    for(std::vector<GLGTTextureData*>::size_type i = 0; i < size; i++) {
        GLGTSAFEDELETE(ptrTextureData[i]);
    }
}

//////////////////////////////////////////////////
bool GLGTTextureManager_Singleton::isTextureExist(const std::string& in_texturename) const {
    std::vector<GLGTTextureData*>::size_type size = ptrTextureData.size();
    for(std::vector<GLGTTextureData*>::size_type i = 0; i < size; i++) {
        if(ptrTextureData[i] != NULL) {
            if(ptrTextureData[i]->name == in_texturename) {
                return true;
            }
        }
    }
    return false;

}

//////////////////////////////////////////////////
GLGTTexture GLGTTextureManager_Singleton::getTexture(const std::string& in_texturename) {
    GLGTTexture ret;
    std::vector<GLGTTextureData*>::size_type size = ptrTextureData.size();
    for(std::vector<GLGTTextureData*>::size_type i = 0; i < size; i++) {
        if(ptrTextureData[i] != NULL) {
            if(ptrTextureData[i]->name == in_texturename) {
                ret.refer(ptrTextureData[i]);
                break;
            }
        }
    }
    return ret;
}

//////////////////////////////////////////////////
GLGTTexture GLGTTextureManager_Singleton::addTexture(const std::string& in_texturename, GLGTTextureType in_type) {
    //łɑ݂ĂȂ`FbN
    if(isTextureExist(in_texturename) == true) {
        return getTexture(in_texturename);
    }

    //eNX`ǉ
    unsigned int index = findUnusedArraySpace();
    ptrTextureData[index] = new GLGTTextureData;
    ptrTextureData[index]->name = in_texturename;
    ptrTextureData[index]->type = in_type;
    ptrTextureData[index]->referenceCount = 0;

    //gqtype𔻒fāAʂtypeɑB
    if(ptrTextureData[index]->type == TEXTURE_UNKNOWN) {
        std::string ext = GLGTUtility.getFileExtensions(ptrTextureData[index]->name);
        if((ext == "tga")||(ext == "TGA")) {
            ptrTextureData[index]->type = TEXTURE_TGA;
        }
    }

    //ǂݍŃ`FbNiǂݍݎsꍇ͍폜)
    GLGTTexture ret;

    if(!loadTexture(*ptrTextureData[index])) {
        GLGTSAFEDELETE(ptrTextureData[index]);
        return ret;
    }

    ret.refer(ptrTextureData[index]);
    return ret;
}

//////////////////////////////////////////////////
// 
GLGTTexture GLGTTextureManager_Singleton::addTexture(const GLGTTextureData& in_texturedata) {
    //łɑ݂ĂȂ`FbN
    if(isTextureExist(in_texturedata.name) == true) {
        return getTexture(in_texturedata.name);
    }

    //eNX`ǉ
    unsigned int index = findUnusedArraySpace();
    ptrTextureData[index] = new GLGTTextureData;
    ptrTextureData[index]->name = in_texturedata.name;
    ptrTextureData[index]->type = in_texturedata.type;
    ptrTextureData[index]->referenceCount = in_texturedata.referenceCount;
    ptrTextureData[index]->width = in_texturedata.width;
    ptrTextureData[index]->height = in_texturedata.height;
    ptrTextureData[index]->pixels = in_texturedata.pixels;

    return GLGTTexture(ptrTextureData[index]);

}

//////////////////////////////////////////////////
void GLGTTextureManager_Singleton::deleteUnreferredData(void) {
    std::vector<GLGTTextureData*>::size_type size = ptrTextureData.size();
    for(std::vector<GLGTTextureData*>::size_type i = 0; i < size; i++) {
        if(ptrTextureData[i] != NULL) {
            if(ptrTextureData[i]->referenceCount <= 0) {
                GLGTSAFEDELETE(ptrTextureData[i]);
            }
        }
    }
}

//////////////////////////////////////////////////
bool GLGTTextureManager_Singleton::loadTexture(GLGTTextureData& in_texturedata) {
    if(in_texturedata.type == TEXTURE_UNKNOWN) {
        return false;
    }

    if(in_texturedata.type == TEXTURE_TGA) {
        return loadTGATexture(in_texturedata);
    }

    return false;

}

//////////////////////////////////////////////////
// 
bool GLGTTextureManager_Singleton::loadTGATexture(GLGTTextureData& in_texturedata) {

    //sNZNAƁAreadDatâ߂width,height0ŏ
    in_texturedata.width = 0;
    in_texturedata.height = 0;
    in_texturedata.pixels.clear();

    // tgat@CI[v 
    std::ifstream tgafile;
    tgafile.open(in_texturedata.name.c_str(), std::ios::binary);
    if (!tgafile.is_open()) {
        return(false);
    }
    




    //wb_ǂݍ

    int err = 0;//G[`FbNp
    unsigned char id_field_length;
    unsigned char color_map_type;
    unsigned char image_type;
    int bpp = 0;// 0ŏȂ΂ȂȂ
    unsigned int alpha_depth;
    bool rle;//rlek
    bool full_color;//24bit or 32bit
    unsigned char ch_buf1;
    unsigned char ch_buf2 = 15; //00001111

    readData(tgafile, (char*)&id_field_length, 1, &err);//ID field length ǂݍ
    readData(tgafile, (char*)&color_map_type, 1, &err);//Color map type ǂݍ
    readData(tgafile, (char*)&image_type, 1, &err); //image type ǂݍ
    tgafile.seekg(5, std::ios::cur);//image origin X ̈ʒuɈړ
    tgafile.seekg(4, std::ios::cur);//image width ̈ʒuɈړ
    readData(tgafile, (char*)&(in_texturedata.width), 2, &err);//image widthǂݍ
    readData(tgafile, (char*)&(in_texturedata.height), 2, &err);//image heightǂݍ
    readData(tgafile, (char*)&bpp, 1, &err);//bbpǂݍ
    readData(tgafile, (char*)&ch_buf1, 1, &err); //discripterǂݍ

    if (err > 0) { //G[`FbN
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }


    //Color map type`FbN
    if (color_map_type == 1) {
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }

    //image type`FbN
    switch (image_type) {
        case 2:
            full_color = true;
            rle = false;
            break;
        case 3:
            full_color = false;
            rle = false;
            break;
        case 10:
            full_color = true;
            rle = true;
            break;
        case 11:
            full_color = false;
            rle = true;
            break;
        default:
            in_texturedata.width = 0;
            in_texturedata.height = 0;
            in_texturedata.pixels.clear();
            return false;
    }
    //k̏ꍇ(蔲)
    if(rle) {
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }
    //bpp`FbN
    if(!((bpp ==  8)||(bpp == 16)||(bpp == 24)||(bpp == 32))) {
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }
    //alpha depth `FbN
    alpha_depth = ch_buf1 & ch_buf2;
    if (! ((alpha_depth == 0) || (alpha_depth == 8))) {
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }




    // f[^ǂݍ(RGBA`Ŋi[)

    //f[^Ɉړ
    tgafile.seekg(id_field_length, std::ios::cur);
    //m
    in_texturedata.pixels.resize(in_texturedata.width*in_texturedata.height*4, 255);//***** RGBA`Ŋi[ *****

    if(!rle) { //k̏ꍇ
        unsigned char buf[4];
        unsigned int size = in_texturedata.width*in_texturedata.height;
        for(unsigned int i = 0; i < size; i++) {
            if(!full_color) { //OCXP[̏ꍇ
                readData(tgafile, (char*)buf, 1, &err);
                buf[1] = buf[0];
                buf[2] = buf[0];
                buf[3] = 255;
            }
            else { //tJ[̏ꍇ
                readData(tgafile, (char*)&buf[2], 1, &err);
                readData(tgafile, (char*)&buf[1], 1, &err);
                readData(tgafile, (char*)&buf[0], 1, &err);
                //alphaL̏ꍇ
                if(alpha_depth == 8) {
                    readData(tgafile, (char*)&buf[3], 1, &err);
                }
                else {
                    buf[3] = 255;
                }
            }
            in_texturedata.pixels[i*4+0] = buf[0];
            in_texturedata.pixels[i*4+1] = buf[1];
            in_texturedata.pixels[i*4+2] = buf[2];
            in_texturedata.pixels[i*4+3] = buf[3];

        }
    }
    else { //kL̏ꍇ
        //蔲
    }



    if (err > 0) { //G[`FbN
        in_texturedata.width = 0;
        in_texturedata.height = 0;
        in_texturedata.pixels.clear();
        return false;
    }

    return true;

}




//////////////////////////////////////////////////
bool GLGTTextureManager_Singleton::outputTextureData(const GLGTTexture& in_texture) const {
    if(in_texture.isReferring() == false) {
        return false;
    }

    std::ofstream outdata;
    std::string outfilename = (std::string)"["+in_texture.getName()+(std::string)"]TextureData.txt";
    outdata.open(outfilename.c_str());
    if(!outdata.is_open()) {
        return false;
    }

    outdata << "#define TEXTUREWIDTH " << in_texture.getWidth() << '\n';
    outdata << "#define TEXTUREHEIGHT " << in_texture.getHeight() << '\n';

    outdata << "\nunsigned char TexturePixel[TEXTUREWIDTH * TEXTUREHEIGHT * 4] = {\n    ";

    char buf[256];

    unsigned int size = in_texture.getWidth()*in_texture.getHeight();
    for(unsigned int i = 0; i < size; i++) {
        if(i != 0) {
            if(i%4 == 0) { outdata << ",\n    ";}
            else { outdata << ", ";}
        }
        sprintf(buf,"0x%.2X,0x%.2X,0x%.2X,0x%.2X",
            (in_texture.getPixels())[i*4  ],(in_texture.getPixels())[i*4+1],
            (in_texture.getPixels())[i*4+2],(in_texture.getPixels())[i*4+3]);
        outdata << buf;
    }

    outdata << "\n};\n" << std::endl;



    return true;



}




//////////////////////////////////////////////////
unsigned int GLGTTextureManager_Singleton::findUnusedArraySpace(void){
    unsigned int size = (unsigned int)ptrTextureData.size();
    for(unsigned int i = 0; i < size; i++) {
        if(ptrTextureData[i] == NULL) {
            return i;
        }
        else if(ptrTextureData[i]->referenceCount <= 0) {
            GLGTSAFEDELETE(ptrTextureData[i]);
            return i;
        }

    }
    ptrTextureData.push_back(NULL);
    return size;
}




